package org.eclipse.jdt.internal.compiler.lookup;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.lang.reflect.Field;
import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.lookup.AnnotationBinding;
import org.eclipse.jdt.internal.compiler.lookup.ProblemReasons;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public class ProblemReferenceBinding extends ReferenceBinding {
  ReferenceBinding closestMatch;
  
  private int problemReason;
  
  public ProblemReferenceBinding(char[][] compoundName, ReferenceBinding closestMatch, int problemReason) {
    this.compoundName = compoundName;
    this.closestMatch = closestMatch;
    this.problemReason = problemReason;
  }
  
  public TypeBinding clone(TypeBinding enclosingType) {
    throw new IllegalStateException();
  }
  
  public TypeBinding closestMatch() {
    return (TypeBinding)this.closestMatch;
  }
  
  public ReferenceBinding closestReferenceMatch() {
    return this.closestMatch;
  }
  
  public boolean hasTypeBit(int bit) {
    if (this.closestMatch != null)
      return this.closestMatch.hasTypeBit(bit); 
    return false;
  }
  
  public int problemId() {
    return this.problemReason;
  }
  
  public static String problemReasonString(int problemReason) {
    try {
      Class<ProblemReasons> reasons = ProblemReasons.class;
      String simpleName = reasons.getName();
      int lastDot = simpleName.lastIndexOf('.');
      if (lastDot >= 0)
        simpleName = simpleName.substring(lastDot + 1); 
      Field[] fields = reasons.getFields();
      for (int i = 0, length = fields.length; i < length; i++) {
        Field field = fields[i];
        if (field.getType().equals(int.class) && 
          field.getInt(reasons) == problemReason)
          return String.valueOf(simpleName) + '.' + field.getName(); 
      } 
    } catch (IllegalAccessException illegalAccessException) {}
    return "unknown";
  }
  
  public void setTypeAnnotations(AnnotationBinding[] annotations, boolean evalNullAnnotations) {}
  
  public char[] shortReadableName() {
    return readableName();
  }
  
  public char[] sourceName() {
    return (this.compoundName.length == 0) ? null : this.compoundName[this.compoundName.length - 1];
  }
  
  public String toString() {
    StringBuffer buffer = new StringBuffer(10);
    buffer.append("ProblemType:[compoundName=");
    buffer.append((this.compoundName == null) ? "<null>" : new String(CharOperation.concatWith(this.compoundName, '.')));
    buffer.append("][problemID=").append(problemReasonString(this.problemReason));
    buffer.append("][closestMatch=");
    buffer.append((this.closestMatch == null) ? "<null>" : this.closestMatch.toString());
    buffer.append("]");
    return buffer.toString();
  }
}
